﻿using System;
using System.Windows.Forms;
using Winsoft.Hid;

namespace Demo
{
    public partial class FormDemo : Form
    {
        public FormDemo()
        {
            InitializeComponent();
        }

        private void AddLine(string line = "")
        {
            if (line.Length == 0 && textBox.Text.Length == 0)
                return; // skip first empty lines
            textBox.AppendText(line + Environment.NewLine);
        }

        private string ToString(byte[] data)
        {
            string result = "";
            for (int i = 0; i < data.Length; ++i)
            {
                if (result.Length > 0)
                    result += ", ";
                result += data[i].ToString();
            }
            return result;
        }

        private readonly HidDevice device = new HidDevice();

        private void FormDemo_Load(object sender, System.EventArgs e)
        {
            device.DeviceArrival += Hid_DeviceArrival;
            device.DeviceRemoved += Hid_DeviceRemoved;
        }

        private void FormDemo_FormClosed(object sender, FormClosedEventArgs e)
        {
            device.Dispose();
        }

        private void Hid_DeviceArrival(object sender, DeviceArrivalEventArgs e)
        {
            AddLine();
            AddLine("Device arrival: " + e.DeviceName);

            try
            {
                device.Open(e.DeviceName, false, false);
                try
                {
                    ShowDeviceInfo();
                }
                finally
                {
                    device.Close();
                }
            }
            catch (Exception ex)
            {
                AddLine("Exception: " + ex.Message);
            }
        }

        private void Hid_DeviceRemoved(object sender, DeviceRemovedEventArgs e)
        {
            AddLine();
            AddLine("Device removed: " + e.DeviceName);
        }

        private void ShowDeviceInfo()
        {
            try
            {
                AddLine("Vendor ID: " + device.VendorID.ToString("X4"));
                AddLine("Product ID: " + device.ProductID.ToString("X4"));
                AddLine("Version number: " + device.VersionNumber.ToString("X4"));
                AddLine("Serial number: " + device.SerialNumber);
                AddLine("Manufacturer: " + device.Manufacturer);
                AddLine("Product: " + device.Product);
                AddLine("Usage page: " + device.UsagePage.ToString());
                AddLine("Usage: " + device.Usage.ToString());
                AddLine("Input report length: " + device.InputReportLength.ToString());
                AddLine("Output report length: " + device.OutputReportLength.ToString());
                AddLine("Feature report length: " + device.FeatureReportLength.ToString());
                AddLine("Input buffer count: " + device.InputBufferCount.ToString());

                for (int i = 1; i < 256; ++i)
                    if (device.HasString(i))
                        AddLine("String " + i.ToString() + ": " + device.GetString(i));
                    else
                        break;
            }
            catch (Exception ex)
            {
                AddLine("Exception: " + ex.Message);
            }
        }

        private void buttonShowHidDevices_Click(object sender, EventArgs e)
        {
            textBox.Clear();

            Cursor.Current = Cursors.WaitCursor;
            try
            {
                string[] deviceNames = device.Enumerate();
                for (int i = 0; i < deviceNames.Length; ++i)
                {
                    AddLine();
                    AddLine("Device: " + deviceNames[i]);

                    try
                    {
                        device.Open(deviceNames[i], false, false);
                        ShowDeviceInfo();

                        if (device.FeatureReportSupported)
                            for (byte j = 0; j < 10; ++j)
                                try
                                {
                                    AddLine("Feature report " + j.ToString() + ": " + ToString(device.GetFeatureReport(j)));
                                }
                                catch (Exception ex)
                                {
                                    AddLine("Feature report " + j.ToString() + " exception: " + ex.Message);
                                }

                        if (device.InputReportSupported)
                            for (byte j = 0; j < 10; ++j)
                                try
                                {
                                    AddLine("Input report " + j.ToString() + ": " + ToString(device.GetInputReport(j)));
                                }
                                catch (Exception ex)
                                {
                                    AddLine("Input report " + j.ToString() + " exception: " + ex.Message);
                                }

                        if (device.ReadSupported)
                        {
                            // reopen device with read access
                            device.Open(deviceNames[i], true, false);
                            for (byte j = 0; j < 10; ++j)
                                try
                                {
                                    AddLine("Read: " + ToString(device.Read(300)));
                                }
                                catch (Exception ex)
                                {
                                    AddLine("Read exception: " + ex.Message);
                                }
                        }
                    }
                    catch (Exception ex)
                    {
                        AddLine("Exception: " + ex.Message);
                    }
                    finally
                    {
                        device.Close();
                    }
                }
            }
            finally
            {
                Cursor.Current = Cursors.Default;
            }
        }
    }
}
