﻿Imports Winsoft.Hid

Public Class FormDemo

    Private Sub AddLine(ByVal Optional line As String = "")
        If line.Length = 0 AndAlso textBox.Text.Length = 0 Then Return
        textBox.AppendText(line & Environment.NewLine)
    End Sub

    Private Function ToString(ByVal data As Byte()) As String
        Dim result As String = ""

        For i As Integer = 0 To data.Length - 1
            If result.Length > 0 Then result += ", "
            result += data(i).ToString()
        Next

        Return result
    End Function

    Private ReadOnly device As HidDevice = New HidDevice()

    Private Sub FormDemo_Load(sender As Object, e As EventArgs) Handles MyBase.Load
        AddHandler device.DeviceArrival, AddressOf Hid_DeviceArrival
        AddHandler device.DeviceRemoved, AddressOf Hid_DeviceRemoved
    End Sub

    Private Sub FormDemo_FormClosed(sender As Object, e As FormClosedEventArgs) Handles MyBase.FormClosed
        device.Dispose()
    End Sub

    Private Sub Hid_DeviceArrival(ByVal sender As Object, ByVal e As DeviceArrivalEventArgs)
        AddLine()
        AddLine("Device arrival: " & e.DeviceName)

        Try
            device.Open(e.DeviceName, False, False)

            Try
                ShowDeviceInfo()
            Finally
                device.Close()
            End Try

        Catch ex As Exception
            AddLine("Exception: " & ex.Message)
        End Try
    End Sub

    Private Sub Hid_DeviceRemoved(ByVal sender As Object, ByVal e As DeviceRemovedEventArgs)
        AddLine()
        AddLine("Device removed: " & e.DeviceName)
    End Sub

    Private Sub ShowDeviceInfo()
        Try
            AddLine("Vendor ID: " & device.VendorID.ToString("X4"))
            AddLine("Product ID: " & device.ProductID.ToString("X4"))
            AddLine("Version number: " & device.VersionNumber.ToString("X4"))
            AddLine("Serial number: " & device.SerialNumber)
            AddLine("Manufacturer: " & device.Manufacturer)
            AddLine("Product: " & device.Product)
            AddLine("Usage page: " & device.UsagePage.ToString())
            AddLine("Usage: " & device.Usage.ToString())
            AddLine("Input report length: " & device.InputReportLength.ToString())
            AddLine("Output report length: " & device.OutputReportLength.ToString())
            AddLine("Feature report length: " & device.FeatureReportLength.ToString())
            AddLine("Input buffer count: " & device.InputBufferCount.ToString())

            For i As Integer = 1 To 255
                If device.HasString(i) Then
                    AddLine("String " & i.ToString() & ": " & device.GetString(i))
                Else
                    Exit For
                End If
            Next

        Catch ex As Exception
            AddLine("Exception: " & ex.Message)
        End Try
    End Sub

    Private Sub buttonShowHidDevices_Click(sender As Object, e As EventArgs) Handles buttonShowHidDevices.Click
        textBox.Clear()
        Cursor.Current = Cursors.WaitCursor

        Try
            Dim deviceNames As String() = device.Enumerate()

            For i As Integer = 0 To deviceNames.Length - 1
                AddLine()
                AddLine("Device: " & deviceNames(i))

                Try
                    device.Open(deviceNames(i), False, False)
                    ShowDeviceInfo()

                    If device.FeatureReportSupported Then
                        For j As Byte = 0 To 9
                            Try
                                AddLine("Feature report " & j.ToString() & ": " & ToString(device.GetFeatureReport(j)))
                            Catch ex As Exception
                                AddLine("Feature report " & j.ToString() & " exception: " & ex.Message)
                            End Try
                        Next
                    End If

                    If device.InputReportSupported Then
                        For j As Byte = 0 To 9
                            Try
                                AddLine("Input report " & j.ToString() & ": " & ToString(device.GetInputReport(j)))
                            Catch ex As Exception
                                AddLine("Input report " & j.ToString() & " exception: " & ex.Message)
                            End Try
                        Next
                    End If

                    If device.ReadSupported Then
                        device.Open(deviceNames(i), True, False)
                        For j As Byte = 0 To 9
                            Try
                                AddLine("Read: " & ToString(device.Read(300)))
                            Catch ex As Exception
                                AddLine("Read exception: " & ex.Message)
                            End Try
                        Next
                    End If
                Catch ex As Exception
                    AddLine("Exception: " & ex.Message)
                Finally
                    device.Close()
                End Try
            Next

        Finally
            Cursor.Current = Cursors.[Default]
        End Try
    End Sub
End Class
